/*
 * fifo based on SRL16 from this white paper:
 * http://www.xilinx.com/support/documentation/white_papers/wp271.pdf
 *
 * modified for SRLC32E
 *
 */

module fifo( clk, in, out, rd, wr, full, empty );

parameter BITS = 32;

input clk;
input  [BITS-1:0] in;
output [BITS-1:0] out;
input rd;
input wr;
output wire full;
output wire empty;

reg [5:0] head = 6'b111111;			// address of first item in fifo 

assign full = head == 6'b011111;  
assign empty = head[5];  

always @(posedge clk)
	if( wr & ~rd)
	    head <= head + 1;
	else if( rd & ~wr )
	    head <= head - 1;

`ifdef SIM
SRLC32E srlc32e_00( .CLK(clk), .CE(wr), .D( in[0]), .Q(out[ 0]), .A(head[4:0]) );
SRLC32E srlc32e_01( .CLK(clk), .CE(wr), .D( in[1]), .Q(out[ 1]), .A(head[4:0]) );
SRLC32E srlc32e_02( .CLK(clk), .CE(wr), .D( in[2]), .Q(out[ 2]), .A(head[4:0]) );
SRLC32E srlc32e_03( .CLK(clk), .CE(wr), .D( in[3]), .Q(out[ 3]), .A(head[4:0]) );
SRLC32E srlc32e_04( .CLK(clk), .CE(wr), .D( in[4]), .Q(out[ 4]), .A(head[4:0]) );
SRLC32E srlc32e_05( .CLK(clk), .CE(wr), .D( in[5]), .Q(out[ 5]), .A(head[4:0]) );
SRLC32E srlc32e_06( .CLK(clk), .CE(wr), .D( in[6]), .Q(out[ 6]), .A(head[4:0]) );
SRLC32E srlc32e_07( .CLK(clk), .CE(wr), .D( in[7]), .Q(out[ 7]), .A(head[4:0]) );
SRLC32E srlc32e_08( .CLK(clk), .CE(wr), .D( in[8]), .Q(out[ 8]), .A(head[4:0]) );
SRLC32E srlc32e_09( .CLK(clk), .CE(wr), .D( in[9]), .Q(out[ 9]), .A(head[4:0]) );
SRLC32E srlc32e_10( .CLK(clk), .CE(wr), .D(in[10]), .Q(out[10]), .A(head[4:0]) );
SRLC32E srlc32e_11( .CLK(clk), .CE(wr), .D(in[11]), .Q(out[11]), .A(head[4:0]) );
SRLC32E srlc32e_12( .CLK(clk), .CE(wr), .D(in[12]), .Q(out[12]), .A(head[4:0]) );
SRLC32E srlc32e_13( .CLK(clk), .CE(wr), .D(in[13]), .Q(out[13]), .A(head[4:0]) );
SRLC32E srlc32e_14( .CLK(clk), .CE(wr), .D(in[14]), .Q(out[14]), .A(head[4:0]) );
SRLC32E srlc32e_15( .CLK(clk), .CE(wr), .D(in[15]), .Q(out[15]), .A(head[4:0]) );
SRLC32E srlc32e_16( .CLK(clk), .CE(wr), .D(in[16]), .Q(out[16]), .A(head[4:0]) );
SRLC32E srlc32e_17( .CLK(clk), .CE(wr), .D(in[17]), .Q(out[17]), .A(head[4:0]) );
SRLC32E srlc32e_18( .CLK(clk), .CE(wr), .D(in[18]), .Q(out[18]), .A(head[4:0]) );
SRLC32E srlc32e_19( .CLK(clk), .CE(wr), .D(in[19]), .Q(out[19]), .A(head[4:0]) );
SRLC32E srlc32e_20( .CLK(clk), .CE(wr), .D(in[20]), .Q(out[20]), .A(head[4:0]) );
SRLC32E srlc32e_21( .CLK(clk), .CE(wr), .D(in[21]), .Q(out[21]), .A(head[4:0]) );
SRLC32E srlc32e_22( .CLK(clk), .CE(wr), .D(in[22]), .Q(out[22]), .A(head[4:0]) );
SRLC32E srlc32e_23( .CLK(clk), .CE(wr), .D(in[23]), .Q(out[23]), .A(head[4:0]) );
SRLC32E srlc32e_24( .CLK(clk), .CE(wr), .D(in[24]), .Q(out[24]), .A(head[4:0]) );
SRLC32E srlc32e_25( .CLK(clk), .CE(wr), .D(in[25]), .Q(out[25]), .A(head[4:0]) );
SRLC32E srlc32e_26( .CLK(clk), .CE(wr), .D(in[26]), .Q(out[26]), .A(head[4:0]) );
SRLC32E srlc32e_27( .CLK(clk), .CE(wr), .D(in[27]), .Q(out[27]), .A(head[4:0]) );
SRLC32E srlc32e_28( .CLK(clk), .CE(wr), .D(in[28]), .Q(out[28]), .A(head[4:0]) );
//SRLC32E srlc32e_29( .CLK(clk), .CE(wr), .D(in[29]), .Q(out[29]), .A(head[4:0]) );
//SRLC32E srlc32e_30( .CLK(clk), .CE(wr), .D(in[30]), .Q(out[30]), .A(head[4:0]) );
//SRLC32E srlc32e_31( .CLK(clk), .CE(wr), .D(in[31]), .Q(out[31]), .A(head[4:0]) );
//SRLC32E srlc32e_32( .CLK(clk), .CE(wr), .D(in[32]), .Q(out[32]), .A(head[4:0]) );
//SRLC32E srlc32e_33( .CLK(clk), .CE(wr), .D(in[33]), .Q(out[33]), .A(head[4:0]) );
//SRLC32E srlc32e_34( .CLK(clk), .CE(wr), .D(in[34]), .Q(out[34]), .A(head[4:0]) );
//SRLC32E srlc32e_35( .CLK(clk), .CE(wr), .D(in[35]), .Q(out[35]), .A(head[4:0]) );

`else
genvar i;
generate
    for( i = 0; i < BITS; i = i + 1 ) begin : srlc32e
      SRLC32E srlc32e(
	.CLK(clk),
	.CE(wr),
	.D(in[i]),
	.Q(out[i]),
	.A(head[4:0]) );
    end
endgenerate
`endif

endmodule

