/*
 * main.v -- main module for 65Org16 DevBoard
 * 
 * 
 * (C) Arlet Ottens, <arlet@c-scape.nl>
 */

`timescale 1ns / 1ps

module main( 
	// clock and reset
	input MAINCLK1, 
	output reg USBCLK, 
	input RESET,
	
	// UART signals
	input RX, 
	output TX,
	
	// I2C signals
	inout SCL,
	inout SDA,

	// debug port
	output [3:0] debug,

	// sdram signals
        inout [15:0] DQ,
	output [12:0] SDRA,  
	output [1:0] SDR1BA,
	output SDR1CSn,
	output SDR1RASn,
	output SDR1CASn,
	output SDR1WEn,
	output SDRCLK,
	output SDRDQM,

	// video signals
	output VIDEOCLK,
	input VSYNC,
	input HSYNC,
        output [7:0] PIXDAT );

wire clk100 = MAINCLK1;		// 100 MHz input clock
wire clk24;			// 24 MHz clock for USB (divided by 2)
wire clk54_0;			//
wire clk54;			// 54 MHz clock for Video (divided by 2)
wire clk; 			// main clock

assign debug = 0;

BUFG BUFG_Main( .I(MAINCLK1), .O(clk) );

//assign clk = clk54;

DCM_CLKGEN #(
	.CLKFX_DIVIDE(100),
	.CLKFX_MD_MAX(0.0), 
	.CLKFX_MULTIPLY(24), 
	.CLKIN_PERIOD("10.0"),
	.STARTUP_WAIT("FALSE") 
)
DCM_CLKGEN_inst (
	.CLKFX(clk24), 
	.CLKIN(clk100), 
	.FREEZEDCM(1'b0), 
	.PROGCLK(1'b0), 
	.PROGDATA(1'b0),
	.PROGEN(1'b0),
	.RST(1'b0)
);

`ifdef SIM

reg clk54_sim = 0;

initial 
    while( 1 )
	#9.259 clk54_sim = ~clk54_sim;

assign clk54_0 = clk54_sim;


`else

wire pll_fb;

PLL_BASE #(
    .BANDWIDTH("LOW"), // "HIGH", "LOW" or "OPTIMIZED"
    .CLKFBOUT_MULT(18), // Multiply value for all CLKOUT clock outputs (1-64)
    .CLKFBOUT_PHASE(0.0), // Phase offset in degrees of the clock feedback output (0.0-360.0).
    .CLKOUT0_DIVIDE(8),
    .CLKOUT1_DIVIDE(8),  
    .CLKOUT2_DIVIDE(8),
    .CLKOUT3_DIVIDE(8),
    .CLKOUT4_DIVIDE(8),
    .CLKOUT5_DIVIDE(8),
    .CLKOUT0_DUTY_CYCLE(0.5),
    .CLKOUT1_DUTY_CYCLE(0.5),
    .CLKOUT2_DUTY_CYCLE(0.5),
    .CLKOUT3_DUTY_CYCLE(0.5),
    .CLKOUT4_DUTY_CYCLE(0.5),
    .CLKOUT5_DUTY_CYCLE(0.5),
    .CLKOUT0_PHASE(0.0),
    .CLKOUT1_PHASE(0.0),
    .CLKOUT2_PHASE(0.0),
    .CLKOUT3_PHASE(0.0),
    .CLKOUT4_PHASE(0.0),
    .CLKOUT5_PHASE(0.0),
    .COMPENSATION("INTERNAL"), // "SYSTEM_SYNCHRONOUS", "SOURCE_SYNCHRONOUS", "EXTERNAL"
    .DIVCLK_DIVIDE(1), // Division value for all output clocks (1-52)
    .REF_JITTER(0.1), // Reference Clock Jitter in UI (0.000-0.999).
    .RESET_ON_LOSS_OF_LOCK("FALSE") // Must be set to FALSE
)
PLL_BASE_inst (
    .CLKFBOUT(pll_fb), // 1-bit output PLL_BASE feedback output
    .CLKOUT0(clk54_0),
    .CLKFBIN(pll_fb), // 1-bit input Feedback clock input
    .CLKIN(clk24), // 1-bit input Clock input
    .RST(1'b0) // 1-bit input Reset input
);

`endif

wire sdram_clk;
wire sdram_clk0;
wire sdram_clk180;

BUFG BUFG_V( .I(clk54_0), .O(clk54) );
BUFG BUFG_1( .I(sdram_clk0), .O(sdram_clk) );
BUFG BUFG_2( .I(sdram_clk180), .O(sdram_clk_inv) );

DCM_SP #(
	 .CLKDV_DIVIDE(2.0),
	 .CLKFX_DIVIDE(1), 
	 .CLKFX_MULTIPLY(2),
	 .CLKIN_DIVIDE_BY_2("FALSE"), 
	 .CLKIN_PERIOD(10.0), 
	 .CLKOUT_PHASE_SHIFT("FIXED"), 
	 .CLK_FEEDBACK("1X"),
	 .DESKEW_ADJUST("SYSTEM_SYNCHRONOUS"), 
	 .DLL_FREQUENCY_MODE("LOW"),
	 .DUTY_CYCLE_CORRECTION("TRUE"), 
`ifdef SIM
	 .PHASE_SHIFT(40), 
`else
	 .PHASE_SHIFT(0), 
`endif
	 .STARTUP_WAIT("FALSE")
) DCM_SP_inst (
	.CLK0(sdram_clk0), 	// 0 degree DCM CLK output
	.CLK180(sdram_clk180), // 180 degree DCM CLK output
	.CLKFB(sdram_clk), 	// DCM clock feedback
	.PSEN(1'b0),		// no variable phase shift
	.CLKIN(MAINCLK1), 	// Clock input (from IBUFG, BUFG or DCM)
	.RST(1'b0) 		
);

ODDR2 ODDR2 (
	.Q(SDRCLK),
	.C0(sdram_clk),
	.C1(sdram_clk_inv),
	.CE(1'b1), 
	.D0(1'b1),
	.D1(1'b0),
	.R(1'b0),
	.S(1'b0) 
);

// divide 24MHz clock to create 12MHz USB clock
always @(posedge clk24)
    USBCLK = ~USBCLK;

// sdram write bus
wire [15:0] sdram_wr_data;
wire sdram_wr_req;
wire sdram_wr_ack;
wire sdram_next_wr_ack;

// sdram read bus
wire [23:0] sdram_addr;
wire [15:0] sdram_rd_data;
wire sdram_rd_req;
wire sdram_rd_valid;
wire sdram_next_rd_valid;
wire sdram_rd_ack;

// uart signals
wire [7:0] uart_rx_data;
wire uart_rx_valid;
wire [7:0] uart_tx_data;
wire uart_tx_valid;
wire uart_tx_ready;

// data input bus
//wire [7:0] DI_SDRAM;
wire [7:0] DI_CS4954;
wire [7:0] DI_WIN;
wire [7:0] DI_UART;
wire [7:0] DI_GPIO;
wor [7:0] DI;

wire [15:0] AB;
wire [7:0] DO;
wire WE;
wire RDY;

// IRQ
wire uart_irq_0;
wire IRQ = uart_irq_0;

wire sel_uart;
wire sel_gpio;
wire sel_window;
wire sel_cs4954;
reg sel_apb;
wire sprites_done;

// VIDEO SDRAM bus

wire [23:0] vidram_addr;
wire [15:0] vidram_rd_data;
wire vidram_rd_req;
wire vidram_rd_ack;
wire vidram_rd_valid;

always @(posedge clk)
    sel_apb <= (AB[15:12] == 4'hb) & ~WE;

// block RAM interface
ram ram( 
	.clk(clk),
	.AB(AB),
	.DO(DI),
	.DI(DO),
	.WE(WE) );

// CPU core
cpu cpu(
	.clk(clk),
	.reset(~RESET),
	.AB(AB),
	.DI(DI),
	.DO(DO),
	.WE(WE),
	.IRQ(IRQ),
	.RDY(RDY),
	.NMI(1'b0));

// simple peripheral interface
apb apb( .clk(clk),
	 .sel(AB[15:12] == 4'hb),
	 .AB(AB[5:4]),
	 .DI(DI),
	 .DI_0(DI_UART),
	 .DI_1(DI_GPIO),
	 .DI_2(DI_WIN),
	 .DI_3(DI_CS4954),
	 .sel_0(sel_uart),
	 .sel_1(sel_gpio),
	 .sel_2(sel_window),
	 .sel_3(sel_cs4954)
	 );

gpio gpio( 
	.clk(clk), 
	.sel(sel_gpio), 
	.AB(AB[0]), 
	.DI(DI_GPIO), 
	.DO(DO), 
	.WE(WE),
	.scl(SCL),
	.sda(SDA) );

uartif uartif0( 
	.clk(clk),
	.sel(sel_uart),
	.AB(AB[1:0]),
	.DI(DO),
	.DO(DI_UART),
	.WE(WE),
	.irq(uart_irq_0),
	.uart_rx_valid(uart_rx_valid),
	.uart_rx_data(uart_rx_data),
	.uart_tx_ready(uart_tx_ready),
	.uart_tx_valid(uart_tx_valid),
	.uart_tx_data(uart_tx_data)
	);

uart uart( 
	.clk(clk),
	.rxd(RX),
	.txd(TX),
	.rx_data(uart_rx_data),
	.rx_valid(uart_rx_valid),
	.tx_data(uart_tx_data),
	.tx_valid(uart_tx_valid),
	.tx_ready(uart_tx_ready) 
	);

sdramif sdramif(
	.clk(clk),
	.AB(AB),
	.WE(WE),
	.DO(DO),
	.DI(DI),
	.RDY(RDY),
	.DI_CTRL(DI_WIN),
	.ctrl(sel_window),

	.vid_addr(vidram_addr),
	.vid_rd_data(vidram_rd_data),
	.vid_rd_req(vidram_rd_req),
	.vid_rd_ack(vidram_rd_ack),
	.vid_rd_valid(vidram_rd_valid),

	.sdram_addr(sdram_addr),
	.sdram_rd_data(sdram_rd_data),
	.sdram_rd_ack(sdram_rd_ack),
	.sdram_next_rd_valid(sdram_next_rd_valid),
	.sdram_rd_valid(sdram_rd_valid),
	.sdram_rd_req(sdram_rd_req),

	.sdram_wr_data(sdram_wr_data),
	.sdram_next_wr_ack(sdram_next_wr_ack),
	.sdram_wr_req(sdram_wr_req)
	);

sdram sdram( 
	.clk(clk),
	.sdram_dq(DQ),
	.sdram_addr(SDRA),
	.sdram_ba(SDR1BA),
	.sdram_cs(SDR1CSn),
	.sdram_ras(SDR1RASn),
	.sdram_cas(SDR1CASn),
	.sdram_we(SDR1WEn),
	.sdram_dqm(SDRDQM),

	.addr(sdram_addr),
	.wr_data(sdram_wr_data),
	.wr_req(sdram_wr_req),
	.wr_ack(sdram_wr_ack),
	.next_wr_ack(sdram_next_wr_ack),

	.rd_data(sdram_rd_data),
	.rd_valid(sdram_rd_valid),
	.next_rd_valid(sdram_next_rd_valid),
	.rd_req(sdram_rd_req),
	.rd_ack(sdram_rd_ack) 
    );

wire vtrigger;
wire odd;
wire interlaced;

wire [17:0] vid_fifo_data;
wire vid_fifo_we;
wire vid_fifo_full;

`ifdef SIM2
sim4954 sim4954(
	.clk54(clk54),
	.pclk(VIDEOCLK),
	.hsync(HSYNC),
	.vsync(VSYNC),
	.yuv(PIXDAT),
	.htrigger(htrigger),
	.vtrigger(vtrigger),
	.odd(odd),
	.interlaced(interlaced),
	.done(sprites_done),

	.clk(clk),

	.fifo_data(vid_fifo_data),
	.fifo_we(vid_fifo_we),
	.fifo_full(vid_fifo_full),

	.sel(sel_cs4954),
	.AB(AB[2:0]),
	.DI(DI_CS4954),
	.DO(DO),
	.WE(WE)
	);
`else
cs4954 cs4954(
	.clk54(clk54),
	.pclk(VIDEOCLK),
	.hsync(HSYNC),
	.vsync(VSYNC),
	.yuv(PIXDAT),
	.htrigger(htrigger),
	.vtrigger(vtrigger),
	.odd(odd),
	.interlaced(interlaced),
	.done(sprites_done),

	.clk(clk),

	.fifo_data(vid_fifo_data),
	.fifo_we(vid_fifo_we),
	.fifo_full(vid_fifo_full),

	.sel(sel_cs4954),
	.AB(AB[2:0]),
	.DI(DI_CS4954),
	.DO(DO),
	.WE(WE)
	);
`endif

sprites sprites(
	.clk(clk),
	.vtrigger(vtrigger),  	// start with new scan
	.odd(odd),
	.interlaced(interlaced),
	.done(sprites_done),

	.vid_fifo_data(vid_fifo_data),
	.vid_fifo_full(vid_fifo_full),
	.vid_fifo_we(vid_fifo_we),

	.ram_addr(vidram_addr),
	.ram_data(vidram_rd_data),
	.ram_rd_req(vidram_rd_req),
	.ram_rd_ack(vidram_rd_ack),
	.ram_rd_valid(vidram_rd_valid),

	// CPU memory interface to tables
	.AB(AB),
	.DO(DO),
	.DI(DI),
	.WE(WE)
	);

endmodule
